#include "KMotionDef.h"

typedef void AUX_FUNCTION(void);
extern AUX_FUNCTION *ServiceAux;
void Idle(void);

estop_test()	// this function monitors for any estop condition
{
	int estopMes = 0;	// used for message displaying after estop kill has occured (message's are blocking)
	// **** Start E-Stop Monitoring ****
	
	// if external estop circuit is lost, or if any other input that should be active disappears, we have an issue
	if (!ReadBit(ESTOP)){
		ClearBit(SYSTEMOK);
		estopMes = ESTRIGGD;
		//printf("External EStop\n");
	}
	// toolchanger should be up/out whenever spindle is enabled
	if(ReadBit(SPDLENA) && (!ReadBit(CRSLUP) || !ReadBit(CRSLOUT))){
		ClearBit(SYSTEMOK);
		estopMes = ESSPDLTC;
	}
	// spindle should never be enabled at the same time as the drawbar is released
	if(ReadBit(SPDLENA) && ReadBit(DRBR)){
		ClearBit(SYSTEMOK);
		estopMes = ESSPDLDBR;
	}
	// if spindle is on, we should have an OK signal, with the channel enabled
	if(persist.UserData[SPDLSTATE] != 0 && (!ReadBit(SPDLOK) || !ch7->Enable)){
		ClearBit(SYSTEMOK);
		estopMes = ESSPDLFAIL;
	}
	// Have any axis drives faulted?
	if(!ReadBit(XFAULT)){
		ClearBit(SYSTEMOK);
		estopMes = ESXFAIL;
	}
	if(!ReadBit(YFAULT)){
		ClearBit(SYSTEMOK);
		estopMes = ESYFAIL;
	}
	if(!ReadBit(ZFAULT)){
		ClearBit(SYSTEMOK);
		estopMes = ESZFAIL;
	}
	// Is the Konnect still there and commuicating?
	if(ServiceAux==Idle){
		ClearBit(SYSTEMOK);
		estopMes = KONFAIL;
	}
	
	// if something has unset the SYSTEMOK bit, then panic
	if(!ReadBit(SYSTEMOK) && !ReadBit(ESTOPHANDLED))
	{
		ClearBit(SPDLENA);				// stop spindle
		Jog(SPINDLE,0);
		persist.UserData[SPDLSTATE] = 0;	// set spindle status to off (otherwise axis disables on re-init due to following error and retriggers e-stop, as axis is still commaned to move)
		ClearBit(STEPENA); // Kill motion
		DisableAxis(0);
		DisableAxis(1);
		DisableAxis(2);
		ClearBit(XHOMED);	// Unhome everything (maybe revise this, as not all faults will result in position lost)
		ClearBit(YHOMED);
		ClearBit(ZHOMED);
		ClearBit(COOLANT);	// Stop coolant
		if(persist.UserData[TOOL_STATE_VAR] != T_IDLE)	// if TC is not idle, set it to ESTOP (will stop motor movement, but retain other output settings)
			persist.UserData[TOOL_STATE_VAR] = T_ESTOP;
		DoPC(PC_COMM_ESTOP);	// finally we'll tell KMotionCNC to EStop
		printf("PC_COMM_ESTOP requested\n"); 
		SetBit(ESTOPHANDLED);	// And set bit so we don't end up stuck in this loop, continually redisplaying MsgBoxes...
		printf("ESTOPHANDLED set\n");
		SetBit(TOWERRED);
		
		// and finally tell the operator why we've just ground to a halt
		switch(estopMes)
			{
				case(ESTRIGGD):
				{
					MsgBoxNoWait("ESTOP!! E-stop, or Limit switch activated",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(ESSPDLTC):
				{
					MsgBoxNoWait("ESTOP!! Spindle enabled while Toolchanger not UP and OUT!",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(ESSPDLDBR):
				{
					MsgBoxNoWait("ESTOP!! Spindle enabled while Drawbar release enabled!",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(ESSPDLFAIL):
				{
					MsgBoxNoWait("ESTOP!! Spindle disabled when on!",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(ESXFAIL):
				{
					MsgBoxNoWait("ESTOP!! X axis faulted",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(ESYFAIL):
				{
					MsgBoxNoWait("ESTOP!! Y axis faulted",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(ESZFAIL):
				{
					MsgBoxNoWait("ESTOP!! Z axis faulted",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				case(KONFAIL):
				{
					MsgBoxNoWait("ESTOP!! Konnect comms failed",MB_OK|MB_ICONEXCLAMATION);
					break;
				}
				
			}
	}
	// **** End E-Stop Monitoring ****
}

main()
{

}
