#include "KMotionDef.h"
#include "Triac.h"
#include "KflopToKMotionCNCFunctions.c"

#define IDLE 0
#define WAITING_RESPONSE 1

int *ChangerState    = &persist.UserData[TOOL_STATE_VAR];
	
// Assumes a Tool Changer State Machine is running in a forever loop
// (which is necessary to allow external buttons to change tools)

//#define TOOL_STATE_VAR 8  	// Tool changer state is saved globally in this Var
//#define TOOL_VAR 9        	// Tool changer desired new tool Var
// define state that the tool changer may be in
//enum {T_IDLE,T_START,T_CRSLIN,T_DBUNCLAMP,T_CRSLDOWN,T_CRSLROTATE,T_CRSLRUN,T_CRSLSTOP,T_WAIT_UNCLAMP,T_WAIT_MOVE,T_WAIT_CLAMP};

main()
{
	//int *ChangerState    = &persist.UserData[TOOL_STATE_VAR];
	int Tool = persist.UserData[TOOL_VAR];  // value stored is an integer 
	int *LastTool 		= &persist.UserData[LAST_TOOL_VAR];
	int *CrslPos		= &persist.UserData[CRSL_POS];
	float awaitingResult = IDLE;
	int Response = FALSE;
	int Answer;

	printf("Carousel Position=%d\n",*CrslPos);

	// Do we have the requested tool already in the spindle?
	if(*LastTool == Tool)
	{
		printf("Tool already in spindle\n");
		ThreadDone();	// we're done, so kill thread
	}
	// is TC not idle?
	if (*ChangerState != T_IDLE)
	{
		printf("ToolChanger not idle\n");
		MsgBox("Tool Changer not idle. Run Initialisation to recover.", MB_OK|MB_ICONEXCLAMATION);	// display message
		// do soft estop to stop any further motion
		ThreadDone();	// and kill thread
	}
	// is spindle enabled?
	if (ReadBit(SPDLENA))
	{
		printf("Spindle still enabled\n");
		DoPC(PC_COMM_HALT);//activate feedhold in KMCNC
		MsgBox("Spindle Enabled. Stop spindle to carry out tool change.", MB_OK|MB_ICONEXCLAMATION); // display message
		// do soft estop to stop any further motion
		ThreadDone();	// and kill thread
	}
	// if LastTool is 0, then TC has not been initialized yet
	if(*LastTool==0){
		printf("Valid tool not set (LastTool=0)\n");
		DoPC(PC_COMM_HALT);//activate feedhold in KMCNC
		MsgBox("No valid tool set. Run Tool Changer Initialisation.",MB_OK|MB_ICONEXCLAMATION);
		// do soft estop to stop any further motion
		ThreadDone();	// and kill thread
	}
	// if Z has not been homed, we can't do a change
	if(!ReadBit(ZHOMED)){
		printf("Z not homed)\n");
		DoPC(PC_COMM_HALT);//activate feedhold in KMCNC
		MsgBox("Z not homed. Home Z and re-try tool change.",MB_OK|MB_ICONEXCLAMATION);
		// do soft estop to stop any further motion
		ThreadDone();	// and kill thread
	}
	
	// just double check we are idle before starting in earnest
	if (*ChangerState == T_IDLE)
	{
		// if we've made it here, then we should be safe to do a tool change.
		printf("Tool %d Change Initiated\n",Tool);
		// so we'll start by setting Z to the correct height
		*ChangerState = T_MOVEZ;  // Go!
		// wait til T_MOVEZ is finished
		while (!tc_test_done())
			WaitNextTimeSlice();
		
		// if LastTool is greater than 6 and less than 99, we have a manual tool in
		if((*LastTool > 6) && (*LastTool < 99))
		{
			printf("Manual tool removal started\n");
			int toolremoved = FALSE;
			while(!toolremoved){
				Answer = MsgBox("Press OK to release tool",MB_OK|MB_ICONEXCLAMATION);
				if(Answer==IDOK){	// if OK, activate drawbar 
					printf("OK\n");
					SetBit(DRBR);
				} else {		// else we want to quit TC Initialisation
					printf("Cancel\n");
				}
							// need to pad this out so we can loop back if cancelled
				Answer = MsgBox("Tool removed?\n(Click No to re-try)",MB_YESNO|MB_ICONEXCLAMATION);
				if(Answer == IDYES){
					toolremoved = TRUE;
				} else {
					ClearBit(DRBR);	// deactivate drawbar and let the loop try again
				}
			}
			*LastTool = 99;
			printf("Manual tool removal complete\n");
		}
		
		// if LastTool is 99, then no tool is in spindle, so we have no tool to remove (it could be in the carousel, so we don't want to knock it out!)
		// do we actually need this?
		if (*LastTool==99){
			printf("No tool in spindle\n");
			//MsgBox("No tool in spindle. STILL TO BE CODED",MB_OK);
		}
		
		// if valid carousel tool set, we do a normal routine to remove tool
		if((*LastTool > 0) && (*LastTool < 7))
		{
			printf("M6 Tool change - CRSLIN\n");
			*ChangerState = T_CRSLIN;  		// Move Carousel In
			while (!tc_test_done())
				WaitNextTimeSlice();
			printf("M6 Tool change - DBUNCLAMP\n");
			*ChangerState = T_DBUNCLAMP;	// Unclamp Drawbar
			while (!tc_test_done())
				WaitNextTimeSlice();
			printf("M6 Tool change - CRSLDOWN\n");
			*ChangerState = T_CRSLDOWN;		// Move carousel down
			while (!tc_test_done())
				WaitNextTimeSlice();
			*LastTool = 99;		// we should now have no tool in spindle, so set LastTool so we know that
		}
		// Tool should now be removed
		
		// is requested tool in the carousel?
		if((Tool > 0) && (Tool < 7)){
			//if last tool was not in carousel, then we need to get the carousel in position
			if(!ReadBit(CRSLDWNR)){
				*ChangerState = T_CRSLDOWN;		// move carousel down
			}
			while (!tc_test_done())
				WaitNextTimeSlice();
			if(!ReadBit(CRSLINR)){			// move carousel in
				*ChangerState = T_CRSLIN;
			}
			while (!tc_test_done())
				WaitNextTimeSlice();
			if(!ReadBit(DRBR)){		// if unclamp relay is not active, then drawbar is not unclamped
				*ChangerState = T_DBUNCLAMP;	// unclamp drawbar
			}
			while (!tc_test_done())
				WaitNextTimeSlice();
			*ChangerState = T_CRSLROTATE;  	// Rotate carousel to required tool
			while (!tc_test_done())
				WaitNextTimeSlice();
			*ChangerState = T_CRSLUP;  		// Move Carousel Up
			while (!tc_test_done())
				WaitNextTimeSlice();
			*ChangerState = T_DBCLAMP;  	// Clamp Drawbar
			while (!tc_test_done())
				WaitNextTimeSlice();
			*ChangerState = T_CRSLOUT;  	// Move carousle out
			while (!tc_test_done())
				WaitNextTimeSlice();
			
			printf("Tool Change Complete\n");
			*LastTool = Tool;  // update current tool in spindle
			*ChangerState = T_IDLE;	// set TC back to idle
			ThreadDone();		// and kill as we're done
		}
		// if Tool is greater than 6, but less than 99, we want a special tool loaded (manual loading needed)
		if((Tool > 6) && (Tool < 99))
		{
			
			printf("Manual tool loading needed\n");
			// we'll make sure the carousel is out/up
			if(ReadBit(CRSLINR)){
				*ChangerState = T_CRSLOUT;		// move carousel out
			}
			while (!tc_test_done())
				WaitNextTimeSlice();
			if(ReadBit(CRSLDWNR)){
				*ChangerState = T_CRSLUP;		// move carousel up
			}
			while (!tc_test_done())
				WaitNextTimeSlice();
			// now we'll start the manual clamping routine
			int toolloaded = FALSE;
			while(!toolloaded){
				if(!ReadBit(DRBR)){	// if drawbar is not unclamped, unclamp it
					*ChangerState = T_DBUNCLAMP;
				}
				while (!tc_test_done())
					WaitNextTimeSlice;
				Answer = MsgBox("Place tool in spindle and press OK to clamp",MB_OK|MB_ICONEXCLAMATION);
				if (Answer == IDOK){
					*ChangerState = T_DBCLAMP;
				while (!tc_test_done())
					WaitNextTimeSlice;
				}
				Answer = MsgBox("Tool loaded and clamped?",MB_YESNO|MB_ICONEXCLAMATION);
				if (Answer == IDYES){
					printf("Tool Loaded OK\n");
					toolloaded = TRUE;
				} else if (Answer == IDNO){	// else we're not done, so let while loop run again
					printf("Tool not loaded, retry\n");
				}
				
			}
			*LastTool = Tool;  // update current tool in spindle
			*ChangerState = T_IDLE;	// set TC back to idle
			printf("Tool Change Complete\n");
			ThreadDone();
			//MsgBox("Manual tool loading needed. STILL TO BE CODED",MB_OK|MB_ICONEXCLAMATION);
		}
		
		// if Tool is 99, then we just need to retract things (any removal should of already happened by now)
		if (Tool == 99){
			printf("We don't want to load a tool\n");
			*ChangerState = T_DBCLAMP;  	// Clamp Drawbar
			while (!tc_test_done())
				WaitNextTimeSlice();
			*ChangerState = T_CRSLOUT;  	// Move Carousel Out (there could be a tool in the slot, so we have to move out first to clear spindle)
			while (!tc_test_done())
				WaitNextTimeSlice();
			*ChangerState = T_CRSLUP;  		// Move Carousel Up
			while (!tc_test_done())
				WaitNextTimeSlice();
			*LastTool = Tool;  // update current tool in spindle
			*ChangerState = T_IDLE;	// set TC back to idle
			printf("Tool Change Complete\n");
		}
		
	}
	else
	{
		printf("Error Tool Changer not idle\n");
		DoPC(PC_COMM_HALT);		//activate feedhold in KMCNC
		ThreadDone();
	}					
}

int tc_test_done()	// tests to see if requested TC step has completed (ChangerState returns to T_PAUSED), or has failed (State goes to T_FAILED)
{
	if(*ChangerState == T_PAUSED)	// Status has gone to paused, then we return TRUE
	{
		return TRUE;
	} else if (*ChangerState == T_FAILED) {	// if we've gone to Failed, we need to halt things
		DoPC(PC_COMM_HALT);//activate feedhold in KMCNC
		printf("ToolChange failure detected during M6\n");
		MsgBox("Tool Change failed. Feedhold activated",MB_OK|MB_ICONEXCLAMATION);
		ThreadDone();
	} else {		// else, we're still waiting for something to happen..
		return FALSE;
	}
}